@@ -1,23 +1,9 @@
 /*
-   Copyright (C) 2010 by Sebastian Trueg <trueg at kde.org>
-   Copyright (C) 2014 by Vishesh Handa <vhanda@kde.org>
+    SPDX-FileCopyrightText: 2010 Sebastian Trueg <trueg at kde.org>
+    SPDX-FileCopyrightText: 2014 Vishesh Handa <vhanda@kde.org>
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of
-   the License or (at your option) version 3 or any later version
-   accepted by the membership of KDE e.V. (or its successor approved
-   by the membership of KDE e.V.), which shall act as a proxy
-   defined in Section 14 of version 3 of the license.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+*/
 
 #include "kinotify.h"
 
@@ -30,6 +16,8 @@
 #include <QDir>
 #include <QTest>
 
+#include <stdio.h>
+
 class KInotifyTest : public QObject
 {
     Q_OBJECT
@@ -39,12 +27,21 @@ private Q_SLOTS:
     void testDeleteFolder();
     void testCreateFolder();
     void testRenameFile();
+    void testRenameDeleteFile();
     void testMoveFile();
     void testRenameFolder();
     void testMoveFolder();
     void testMoveFromUnwatchedFolder();
+    void testMoveToUnwatchedFolder();
     void testMoveRootFolder();
     void testFileClosedAfterWrite();
+
+    void init();
+
+private:
+    std::unique_ptr<QTemporaryDir> m_tmpDir;
+    QDir m_dir;
+    std::unique_ptr<KInotify> m_kn;
 };
 
 namespace
@@ -64,19 +61,29 @@ void mkdir(const QString& path)
 }
 }
 
+void KInotifyTest::init()
+{
+    m_tmpDir = std::make_unique<QTemporaryDir>();
+
+    const QString subDir(QStringLiteral("%1/subdir/").arg(m_tmpDir->path()));
+    mkdir(subDir);
+    m_dir = QDir(subDir);
+
+    m_kn = std::make_unique<KInotify>(nullptr);
+    QVERIFY(m_kn->addWatch(m_dir.path(), KInotify::EventAll));
+
+    QSignalSpy kiSpy(m_kn.get(), &KInotify::installedWatches);
+    QVERIFY(kiSpy.count() || kiSpy.wait());
+}
+
 void KInotifyTest::testDeleteFile()
 {
     // create some test files
-    QTemporaryDir dir;
-    const QString f1(QStringLiteral("%1/randomJunk1").arg(dir.path()));
+    const QString f1(QStringLiteral("%1/randomJunk1").arg(m_dir.path()));
     touchFile(f1);
 
-    // start the inotify watcher
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(dir.path(), KInotify::EventAll);
-
     // listen to the desired signal
-    QSignalSpy spy(&kn, SIGNAL(deleted(QString,bool)));
+    QSignalSpy spy(m_kn.get(), &KInotify::deleted);
 
     // test removing a file
     QFile::remove(f1);
@@ -88,47 +95,34 @@ void KInotifyTest::testDeleteFile()
 
 void KInotifyTest::testDeleteFolder()
 {
-    // create some test files
-    QTemporaryDir dir;
-    const QString d1(QStringLiteral("%1/randomJunk4/").arg(dir.path()));
-    mkdir(d1);
-
-    // start the inotify watcher
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(dir.path(), KInotify::EventAll);
+    const QString d1(QStringLiteral("%1/").arg(m_dir.path()));
 
     // listen to the desired signal
-    QSignalSpy spy(&kn, SIGNAL(deleted(QString,bool)));
+    QSignalSpy spy(m_kn.get(), &KInotify::deleted);
 
     // test removing a folder
-    QDir().rmdir(d1);
+    QVERIFY(QDir().rmdir(d1));
     QVERIFY(spy.wait());
     QCOMPARE(spy.count(), 1);
     QCOMPARE(spy.first().at(0).toString(), d1);
     QCOMPARE(spy.first().at(1).toBool(), true);
     // make sure we do not watch the removed folder anymore
-    QVERIFY(!kn.watchingPath(d1));
+    QVERIFY(!m_kn->watchingPath(d1));
 }
 
 
 void KInotifyTest::testCreateFolder()
 {
-    QTemporaryDir dir;
-
-    // start the inotify watcher
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(dir.path(), KInotify::EventAll);
-
     // listen to the desired signal
-    QSignalSpy createdSpy(&kn, SIGNAL(created(QString,bool)));
+    QSignalSpy createdSpy(m_kn.get(), &KInotify::created);
 
     // create the subdir
-    const QString d1(QStringLiteral("%1/randomJunk1/").arg(dir.path()));
+    const QString d1(QStringLiteral("%1/randomJunk1/").arg(m_dir.path()));
     mkdir(d1);
     QVERIFY(createdSpy.wait());
     QCOMPARE(createdSpy.count(), 1);
     QCOMPARE(createdSpy.takeFirst().at(0).toString(), d1);
-    QVERIFY(kn.watchingPath(d1));
+    QVERIFY(m_kn->watchingPath(d1));
 
     // lets go one level deeper
     const QString d2 = QStringLiteral("%1subdir1/").arg(d1);
@@ -136,7 +130,7 @@ void KInotifyTest::testCreateFolder()
     QVERIFY(createdSpy.wait());
     QCOMPARE(createdSpy.count(), 1);
     QCOMPARE(createdSpy.takeFirst().at(0).toString(), d2);
-    QVERIFY(kn.watchingPath(d2));
+    QVERIFY(m_kn->watchingPath(d2));
 
     // although we are in the folder test lets try creating a file
     const QString f1 = QStringLiteral("%1somefile1").arg(d2);
@@ -150,20 +144,15 @@ void KInotifyTest::testCreateFolder()
 void KInotifyTest::testRenameFile()
 {
     // create some test files
-    QTemporaryDir dir;
-    const QString f1(QStringLiteral("%1/randomJunk1").arg(dir.path()));
+    const QString f1(QStringLiteral("%1/randomJunk1").arg(m_dir.path()));
     touchFile(f1);
 
-    // start the inotify watcher
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(dir.path(), KInotify::EventAll);
-
     // listen to the desired signal
-    QSignalSpy spy(&kn, SIGNAL(moved(QString,QString)));
+    QSignalSpy spy(m_kn.get(), &KInotify::moved);
 
     // actually move the file
-    const QString f2(QStringLiteral("%1/randomJunk2").arg(dir.path()));
-    QFile::rename(f1, f2);
+    const QString f2(QStringLiteral("%1/randomJunk2").arg(m_dir.path()));
+    rename(f1.toLocal8Bit().constData(), f2.toLocal8Bit().constData());
 
     // check the desired signal
     QVERIFY(spy.wait());
@@ -173,8 +162,8 @@ void KInotifyTest::testRenameFile()
     QCOMPARE(args.at(1).toString(), f2);
 
     // test a subsequent rename
-    const QString f3(QStringLiteral("%1/randomJunk3").arg(dir.path()));
-    QFile::rename(f2, f3);
+    const QString f3(QStringLiteral("%1/randomJunk3").arg(m_dir.path()));
+    rename(f2.toLocal8Bit().constData(), f3.toLocal8Bit().constData());
 
     // check the desired signal
     QVERIFY(spy.wait());
@@ -184,26 +173,56 @@ void KInotifyTest::testRenameFile()
     QCOMPARE(args.at(1).toString(), f3);
 }
 
+void KInotifyTest::testRenameDeleteFile()
+{
+    // create some test files
+    const QString f1(QStringLiteral("%1/randomJunk1").arg(m_dir.path()));
+    touchFile(f1);
+
+    // listen to the desired signal
+    QSignalSpy spy(m_kn.get(), &KInotify::moved);
+
+    // actually move the file
+    const QString f2(QStringLiteral("%1/randomJunk2").arg(m_dir.path()));
+    rename(f1.toLocal8Bit().constData(), f2.toLocal8Bit().constData());
+
+    // check the desired signal
+    QVERIFY(spy.wait());
+    QCOMPARE(spy.count(), 1);
+    QList<QVariant> args = spy.takeFirst();
+    QCOMPARE(args.at(0).toString(), f1);
+    QCOMPARE(args.at(1).toString(), f2);
+
+    // test a subsequent delete
+    QSignalSpy spy1(m_kn.get(), &KInotify::deleted);
+    QFile::remove(f2);
+
+    // check the desired signal
+    QVERIFY(spy1.wait());
+    QCOMPARE(spy1.count(), 1);
+    QCOMPARE(spy1.takeFirst().at(0).toString(), f2);
+
+    QVERIFY(spy.isEmpty());
+}
 
 void KInotifyTest::testMoveFile()
 {
     // create some test files
-    QTemporaryDir dir1;
-    QTemporaryDir dir2;
-    const QString src(QStringLiteral("%1/randomJunk1").arg(dir1.path()));
-    const QString dest(QStringLiteral("%1/randomJunk2").arg(dir2.path()));
+    QTemporaryDir destDir;
+    const QString src(QStringLiteral("%1/randomJunk1").arg(m_dir.path()));
+    const QString dest(QStringLiteral("%1/randomJunk2").arg(destDir.path()));
     touchFile(src);
 
-    // start the inotify watcher
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(dir1.path(), KInotify::EventAll);
-    kn.addWatch(dir2.path(), KInotify::EventAll);
+    // Add another path without common parent to the watcher
+    QSignalSpy knSpy(m_kn.get(), &KInotify::installedWatches);
+    m_kn->addWatch(destDir.path(), KInotify::EventAll);
+    QVERIFY(knSpy.count() || knSpy.wait());
 
     // listen to the desired signal
-    QSignalSpy spy(&kn, SIGNAL(moved(QString,QString)));
+    QSignalSpy spy(m_kn.get(), &KInotify::moved);
 
     // actually move the file
-    QFile::rename(src, dest);
+    rename(src.toLocal8Bit().constData(), dest.toLocal8Bit().constData());
 
     // check the desired signal
     QVERIFY(spy.wait());
@@ -213,8 +232,8 @@ void KInotifyTest::testMoveFile()
     QCOMPARE(args.at(1).toString(), dest);
 
     // test a subsequent move (back to the original folder)
-    const QString dest2(QStringLiteral("%1/randomJunk3").arg(dir1.path()));
-    QFile::rename(dest, dest2);
+    const QString dest2(QStringLiteral("%1/randomJunk3").arg(m_dir.path()));
+    rename(dest.toLocal8Bit().constData(), dest2.toLocal8Bit().constData());
 
     // check the desired signal
     QVERIFY(spy.wait());
@@ -224,24 +243,23 @@ void KInotifyTest::testMoveFile()
     QCOMPARE(args.at(1).toString(), dest2);
 }
 
-
 void KInotifyTest::testRenameFolder()
 {
     // create some test files
-    QTemporaryDir dir;
-    const QString d1(QStringLiteral("%1/randomJunk1/").arg(dir.path()));
+    const QString d1(QStringLiteral("%1/randomJunk1/").arg(m_dir.path()));
     mkdir(d1);
 
-    // start the inotify watcher
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(dir.path(), KInotify::EventAll);
+    {
+	QSignalSpy spy(m_kn.get(), &KInotify::created);
+	QVERIFY(spy.wait());
+    }
 
     // listen to the desired signal
-    QSignalSpy spy(&kn, SIGNAL(moved(QString,QString)));
+    QSignalSpy spy(m_kn.get(), &KInotify::moved);
 
     // actually rename the folder
-    const QString d2(QStringLiteral("%1/randomJunk2/").arg(dir.path()));
-    QFile::rename(d1, d2);
+    const QString d2(QStringLiteral("%1/randomJunk2/").arg(m_dir.path()));
+    rename(d1.toLocal8Bit().constData(), d2.toLocal8Bit().constData());
 
     // check the desired signal
     QVERIFY(spy.wait());
@@ -251,12 +269,12 @@ void KInotifyTest::testRenameFolder()
     QCOMPARE(args.at(1).toString(), d2);
 
     // check the path cache
-    QVERIFY(!kn.watchingPath(d1));
-    QVERIFY(kn.watchingPath(d2));
+    QVERIFY(!m_kn->watchingPath(d1));
+    QVERIFY(m_kn->watchingPath(d2));
 
     // test a subsequent rename
-    const QString d3(QStringLiteral("%1/randomJunk3/").arg(dir.path()));
-    QFile::rename(d2, d3);
+    const QString d3(QStringLiteral("%1/randomJunk3/").arg(m_dir.path()));
+    rename(d2.toLocal8Bit().constData(), d3.toLocal8Bit().constData());
 
     // check the desired signal
     QVERIFY(spy.wait());
@@ -266,9 +284,9 @@ void KInotifyTest::testRenameFolder()
     QCOMPARE(args.at(1).toString(), d3);
 
     // check the path cache
-    QVERIFY(!kn.watchingPath(d1));
-    QVERIFY(!kn.watchingPath(d2));
-    QVERIFY(kn.watchingPath(d3));
+    QVERIFY(!m_kn->watchingPath(d1));
+    QVERIFY(!m_kn->watchingPath(d2));
+    QVERIFY(m_kn->watchingPath(d3));
 
 
     // KInotify claims it has updated its data structures, lets see if that is true
@@ -276,7 +294,7 @@ void KInotifyTest::testRenameFolder()
     // listen to the desired signal
     const QString f4(QStringLiteral("%1somefile").arg(d3));
 
-    QSignalSpy createdSpy(&kn, SIGNAL(created(QString,bool)));
+    QSignalSpy createdSpy(m_kn.get(), &KInotify::created);
 
     // test creating a file
     touchFile(f4);
@@ -286,26 +304,24 @@ void KInotifyTest::testRenameFolder()
     QCOMPARE(createdSpy.takeFirst().at(0).toString(), f4);
 }
 
-
 void KInotifyTest::testMoveFolder()
 {
     // create some test files
-    QTemporaryDir dir1;
-    QTemporaryDir dir2;
-    const QString src(QStringLiteral("%1/randomJunk1/").arg(dir1.path()));
-    const QString dest(QStringLiteral("%1/randomJunk2/").arg(dir2.path()));
+    QTemporaryDir destDir;
+    const QString src(QStringLiteral("%1/randomJunk1/").arg(m_dir.path()));
+    const QString dest(QStringLiteral("%1/randomJunk2/").arg(destDir.path()));
     mkdir(src);
 
-    // start the inotify watcher
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(dir1.path(), KInotify::EventAll);
-    kn.addWatch(dir2.path(), KInotify::EventAll);
+    // Add another path without common parent to the watcher
+    QSignalSpy knSpy(m_kn.get(), &KInotify::installedWatches);
+    m_kn->addWatch(destDir.path(), KInotify::EventAll);
+    QVERIFY(knSpy.count() || knSpy.wait());
 
     // listen to the desired signal
-    QSignalSpy spy(&kn, SIGNAL(moved(QString,QString)));
+    QSignalSpy spy(m_kn.get(), &KInotify::moved);
 
     // actually move the file
-    QFile::rename(src, dest);
+    rename(src.toLocal8Bit().constData(), dest.toLocal8Bit().constData());
 
     // check the desired signal
     QVERIFY(spy.wait());
@@ -315,12 +331,12 @@ void KInotifyTest::testMoveFolder()
     QCOMPARE(args.at(1).toString(), dest);
 
     // check the path cache
-    QVERIFY(!kn.watchingPath(src));
-    QVERIFY(kn.watchingPath(dest));
+    QVERIFY(!m_kn->watchingPath(src));
+    QVERIFY(m_kn->watchingPath(dest));
 
     // test a subsequent move
-    const QString dest2(QStringLiteral("%1/randomJunk3/").arg(dir1.path()));
-    QFile::rename(dest, dest2);
+    const QString dest2(QStringLiteral("%1/randomJunk3/").arg(m_dir.path()));
+    rename(dest.toLocal8Bit().constData(), dest2.toLocal8Bit().constData());
 
     // check the desired signal
     QVERIFY(spy.wait());
@@ -330,9 +346,9 @@ void KInotifyTest::testMoveFolder()
     QCOMPARE(args.at(1).toString(), dest2);
 
     // check the path cache
-    QVERIFY(!kn.watchingPath(src));
-    QVERIFY(!kn.watchingPath(dest));
-    QVERIFY(kn.watchingPath(dest2));
+    QVERIFY(!m_kn->watchingPath(src));
+    QVERIFY(!m_kn->watchingPath(dest));
+    QVERIFY(m_kn->watchingPath(dest2));
 
 
     // KInotify claims it has updated its data structures, lets see if that is true
@@ -340,7 +356,7 @@ void KInotifyTest::testMoveFolder()
     // listen to the desired signal
     const QString f4(QStringLiteral("%1somefile").arg(dest2));
 
-    QSignalSpy createdSpy(&kn, SIGNAL(created(QString,bool)));
+    QSignalSpy createdSpy(m_kn.get(), &KInotify::created);
 
     // test creating a file
     touchFile(f4);
@@ -352,17 +368,12 @@ void KInotifyTest::testMoveFolder()
 
 void KInotifyTest::testMoveFromUnwatchedFolder()
 {
-    // create some test folders
-    QTemporaryDir dir;
-    const QString src(QStringLiteral("%1/randomJunk1").arg(dir.path()));
-    const QString dest(QStringLiteral("%1/randomJunk2").arg(dir.path()));
-    mkdir(src);
-    mkdir(dest);
+    // create unwatched source folder
+    QTemporaryDir srcDir;
+    const QString src{srcDir.path()};
+    const QString dest{m_dir.path()};
 
-    // Start watching only for destination
-    KInotify kn(nullptr);
-    kn.addWatch(dest, KInotify::EventAll);
-    QSignalSpy spy(&kn, &KInotify::created);
+    QSignalSpy spy(m_kn.get(), &KInotify::created);
 
     // Create stuff inside src
     mkdir(QStringLiteral("%1/sub").arg(src));
@@ -378,7 +389,7 @@ void KInotifyTest::testMoveFromUnwatchedFolder()
     QCOMPARE(spy.count(), 4);
 
     // Checking if watches are installed
-    QSignalSpy spy1(&kn, &KInotify::deleted);
+    QSignalSpy spy1(m_kn.get(), &KInotify::deleted);
     QDir dstdir(QStringLiteral("%1/sub").arg(dest));
     dstdir.removeRecursively();
 
@@ -386,23 +397,47 @@ void KInotifyTest::testMoveFromUnwatchedFolder()
     QCOMPARE(spy1.count(), 4);
 }
 
+void KInotifyTest::testMoveToUnwatchedFolder()
+{
+    // create unwatched destination folder
+    QTemporaryDir destDir;
+    const QString src{m_dir.path()};
+    const QString dest{destDir.path()};
+
+    QSignalSpy spy(m_kn.get(), &KInotify::created);
+
+    // Create stuff inside src
+    mkdir(QStringLiteral("%1/sub").arg(src));
+    touchFile(QStringLiteral("%1/sub/file1").arg(src));
+    touchFile(QStringLiteral("%1/file2").arg(src));
+
+    QVERIFY(spy.wait());
+    QCOMPARE(spy.count(), 3);
+
+    // Move file
+    QFile::rename(QStringLiteral("%1/file2").arg(src),
+            QStringLiteral("%1/file2").arg(dest));
+
+    QSignalSpy spy1(m_kn.get(), &KInotify::deleted);
+    QVERIFY(spy1.wait());
+    QCOMPARE(spy1.count(), 1);
+
+    // Move dir
+    QFile::rename(QStringLiteral("%1/sub").arg(src),
+            QStringLiteral("%1/sub").arg(dest));
+
+    QVERIFY(spy1.wait());
+    QCOMPARE(spy1.count(), 2);
+}
 
 void KInotifyTest::testMoveRootFolder()
 {
-    // create some test folders
-    QTemporaryDir dir;
-    const QString src(QStringLiteral("%1/randomJunk1").arg(dir.path()));
-    const QString dest(QStringLiteral("%1/randomJunk2").arg(dir.path()));
-    mkdir(src);
-
-    // start watching the new subfolder only
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(src, KInotify::EventAll);
-
     // listen for the moved signal
-    QSignalSpy spy(&kn, SIGNAL(moved(QString,QString)));
+    QSignalSpy spy(m_kn.get(), &KInotify::moved);
 
-    // actually move the file
+    // Rename the watched root directory
+    const QString src(QStringLiteral("%1/").arg(m_dir.path()));
+    const QString dest(QStringLiteral("%1/newname/").arg(m_tmpDir->path()));
     QFile::rename(src, dest);
 
     // check the desired signal
@@ -414,24 +449,18 @@ void KInotifyTest::testMoveRootFolder()
     QCOMPARE(args.at(1).toString(), dest);
 
     // check the path cache
-    QVERIFY(!kn.watchingPath(src));
-    QVERIFY(kn.watchingPath(dest));
+    QVERIFY(!m_kn->watchingPath(src));
+    QVERIFY(m_kn->watchingPath(dest));
 }
 
 void KInotifyTest::testFileClosedAfterWrite()
 {
-    QTemporaryDir dir;
-    touchFile(dir.path() + QLatin1String("/file"));
-
-    KInotify kn(nullptr /*no config*/);
-    kn.addWatch(dir.path(), KInotify::EventAll);
-
-    QSignalSpy spy(&kn, SIGNAL(closedWrite(QString)));
-    touchFile(dir.path() + QLatin1String("/file"));
+    QSignalSpy spy(m_kn.get(), &KInotify::closedWrite);
+    touchFile(m_dir.path() + QLatin1String("/file"));
 
     QVERIFY(spy.wait());
     QCOMPARE(spy.count(), 1);
-    QCOMPARE(spy.at(0).first().toString(), QString(dir.path() + QLatin1String("/file")));
+    QCOMPARE(spy.at(0).first().toString(), QString(m_dir.path() + QLatin1String("/file")));
 }
 
 
