@@ -76,6 +76,7 @@ const char* const s_defaultFileExcludeFilters[] = {
     "*.sql",     // SQL database dumps
     "*.sql.gz",  // Compressed SQL database dumps
     "*.ytdl",    // youtube-dl temp files
+    "*.tfstate*", // Terraform state files
 
     // Bytecode files
     "*.class", // Java
@@ -98,7 +99,7 @@ const char* const s_defaultFileExcludeFilters[] = {
     nullptr
 };
 
-const int s_defaultFileExcludeFiltersVersion = 8;
+const int s_defaultFileExcludeFiltersVersion = 9;
 
 const char* const s_defaultFolderExcludeFilters[] = {
     "po",
@@ -126,6 +127,9 @@ const char* const s_defaultFolderExcludeFilters[] = {
     "node_modules",
     "node_packages",
     "nbproject",
+    ".terraform",
+    ".venv",
+    "venv",
 
     //misc
     "core-dumps",
@@ -135,7 +139,7 @@ const char* const s_defaultFolderExcludeFilters[] = {
     nullptr
 };
 
-const int s_defaultFolderExcludeFiltersVersion = 2;
+const int s_defaultFolderExcludeFiltersVersion = 4;
 
 const char* const s_sourceCodeMimeTypes[] = {
     "text/css",
@@ -229,4 +233,3 @@ int Baloo::defaultExcludeMimetypesVersion()
     // The +1 is the image, video and audio mimetypes
     return s_sourceCodeMimeTypesVersion + 1;
 }
-
