@@ -55,7 +55,6 @@ public:
     Private(KInotify* parent)
         : userLimitReachedSignaled(false)
         , config(nullptr)
-        , m_dirIter(nullptr)
         , m_inotifyFd(-1)
         , m_notifier(nullptr)
         , q(parent) {
@@ -63,7 +62,6 @@ public:
 
     ~Private() {
         close();
-        delete m_dirIter;
     }
 
     struct MovedFileCookie {
@@ -81,9 +79,19 @@ public:
     QHash<int, QByteArray> watchPathHash;
     QHash<QByteArray, int> pathWatchHash;
 
+    bool parentWatched(const QByteArray& path)
+    {
+        auto parent = path.chopped(1);
+        if (auto index = parent.lastIndexOf('/'); index > 0) {
+            parent.truncate(index + 1);
+            return pathWatchHash.contains(parent);
+        }
+        return false;
+    }
+
     Baloo::FileIndexerConfig* config;
     QStringList m_paths;
-    Baloo::FilteredDirIterator* m_dirIter;
+    std::unique_ptr<Baloo::FilteredDirIterator> m_dirIter;
 
     // FIXME: only stored from the last addWatch call
     WatchEvents mode;
@@ -139,41 +147,35 @@ public:
     /**
      * Add one watch and call oneself asynchronously
      */
-    bool _k_addWatches() {
-        bool addedWatchSuccessfully = false;
-
-        //Do nothing if the inotify user limit has been signaled.
-        if (userLimitReachedSignaled) {
-            return false;
-        }
-
+    void _k_addWatches()
+    {
         // It is much faster to add watches in batches instead of adding each one
         // asynchronously. Try out the inotify test to compare results.
         for (int i = 0; i < 1000; i++) {
+            if (userLimitReachedSignaled) {
+                return;
+            }
             if (!m_dirIter || m_dirIter->next().isEmpty()) {
                 if (!m_paths.isEmpty()) {
-                    delete m_dirIter;
-                    m_dirIter = new Baloo::FilteredDirIterator(config, m_paths.takeFirst(), Baloo::FilteredDirIterator::DirsOnly);
+                    m_dirIter = std::make_unique<Baloo::FilteredDirIterator>(config, m_paths.takeFirst(), Baloo::FilteredDirIterator::DirsOnly);
                 } else {
-                    delete m_dirIter;
                     m_dirIter = nullptr;
                     break;
                 }
             } else {
-                QString path = m_dirIter->filePath();
-                addedWatchSuccessfully = addWatch(path);
+                addWatch(m_dirIter->filePath());
             }
         }
 
         // asynchronously add the next batch
         if (m_dirIter) {
-            QMetaObject::invokeMethod(q, "_k_addWatches", Qt::QueuedConnection);
+            QMetaObject::invokeMethod(q, [this]() {
+                this->_k_addWatches();
+            }, Qt::QueuedConnection);
         }
         else {
             Q_EMIT q->installedWatches();
         }
-
-        return addedWatchSuccessfully;
     }
 
 private:
@@ -214,34 +216,6 @@ KInotify::~KInotify()
     delete d;
 }
 
-
-bool KInotify::available() const
-{
-    if (d->inotify() > 0) {
-        // trueg: Copied from KDirWatch.
-        struct utsname uts;
-        int major;
-        int minor;
-        int patch = 0;
-        if (uname(&uts) < 0) {
-            return false; // *shrug*
-        } else if (sscanf(uts.release, "%d.%d.%d", &major, &minor, &patch) != 3) {
-            //Kernels > 3.0 can in principle have two-number versions.
-            if (sscanf(uts.release, "%d.%d", &major, &minor) != 2) {
-                return false; // *shrug*
-            }
-        } else if (major * 1000000 + minor * 1000 + patch < 2006014) { // <2.6.14
-            qCDebug(BALOO) << "Can't use INotify, Linux kernel too old";
-            return false;
-        }
-
-        return true;
-    } else {
-        return false;
-    }
-}
-
-
 bool KInotify::watchingPath(const QString& path) const
 {
     const QByteArray p = normalizeTrailingSlash(QFile::encodeName(path));
@@ -259,15 +233,17 @@ bool KInotify::addWatch(const QString& path, WatchEvents mode, WatchFlags flags)
 
     d->mode = mode;
     d->flags = flags;
+    d->m_paths << path;
     // If the inotify user limit has been signaled,
-    // just queue this folder for watching.
-    if (d->m_dirIter || d->userLimitReachedSignaled) {
-        d->m_paths << path;
+    // there will be no watchInstalled signal
+    if (d->userLimitReachedSignaled) {
         return false;
     }
 
-    d->m_dirIter = new Baloo::FilteredDirIterator(d->config, path, Baloo::FilteredDirIterator::DirsOnly);
-    return d->_k_addWatches();
+    QMetaObject::invokeMethod(this, [this]() {
+        this->d->_k_addWatches();
+    }, Qt::QueuedConnection);
+    return true;
 }
 
 
@@ -282,7 +258,6 @@ bool KInotify::removeWatch(const QString& path)
     }
     if (d->m_dirIter) {
         if (d->m_dirIter->filePath().startsWith(path)) {
-            delete d->m_dirIter;
             d->m_dirIter = nullptr;
         }
     }
@@ -414,7 +389,8 @@ void KInotify::slotEvent(int socket)
 //            qCDebug(BALOO) << path << "EventMoveSelf";
             // Problematic if the parent is not watched, otherwise
             // handled by MoveFrom/MoveTo from the parent
-            qCWarning(BALOO) << path << "EventMoveSelf: THIS CASE MAY NOT BE HANDLED PROPERLY!";
+            if (!d->parentWatched(path))
+                qCWarning(BALOO) << path << "EventMoveSelf: THIS CASE MAY NOT BE HANDLED PROPERLY!";
         }
         if (event->mask & EventMoveFrom) {
 //            qCDebug(BALOO) << path << "EventMoveFrom";
