@@ -13,7 +13,6 @@
 
 #include <QString>
 #include <QStringList>
-#include <QSharedPointer>
 #include <QUrlQuery>
 
 #include <QJsonDocument>
@@ -23,7 +22,7 @@ using namespace Baloo;
 
 const int defaultLimit = -1;
 
-class Baloo::Query::Private {
+class BALOO_CORE_NO_EXPORT Baloo::Query::Private {
 public:
     Term m_term;
 
@@ -50,10 +49,7 @@ Query::Query(const Query& rhs)
 {
 }
 
-Query::~Query()
-{
-    delete d;
-}
+Query::~Query() = default;
 
 void Query::addType(const QString& type)
 {
